@@ -9,7 +9,7 @@
 
 Usage:
   kibot [-b BOARD] [-e SCHEMA] [-c CONFIG] [-d OUT_DIR] [-s PRE]
-         [-q | -v...] [-i] [TARGET...]
+         [-q | -v...] [-i] [-g DEF]... [TARGET...]
   kibot [-v...] [-c PLOT_CONFIG] --list
   kibot [-v...] [-b BOARD] [-d OUT_DIR] [-p | -P] --example
   kibot [-v...] --help-list-outputs
@@ -28,6 +28,7 @@ Options:
   -c CONFIG, --plot-config CONFIG  The plotting config file to use
   -d OUT_DIR, --out-dir OUT_DIR    The output directory [default: .]
   -e SCHEMA, --schematic SCHEMA    The schematic file (.sch)
+  -g DEF, --global-redef DEF       Overwrite a global value (VAR=VAL)
   --help-list-outputs              List supported outputs
   --help-output HELP_OUTPUT        Help for this particular output
   --help-outputs                   List supported outputs and details
@@ -50,7 +51,7 @@ __license__ = 'GPL v3+'
 __email__ = 'set@ieee.org'
 __url__ = 'https://github.com/INTI-CMNB/KiBot/'
 __status__ = 'stable'
-__version__ = '0.6.2'
+__version__ = '0.7.0'
 
 
 import os
@@ -72,35 +73,41 @@ from .misc import (NO_PCB_FILE, NO_SCH_FILE, EXIT_BAD_ARGS)
 from .docopt import docopt
 
 logger = None
-has_macro = ['pre_filters',
-             'out_any_layer',
-             'out_svg_sch_print',
-             'pre_erc',
-             'out_gerb_drill',
+has_macro = [
+             'layer',
              'drill_marks',
-             'pre_update_xml',
-             'out_pdf_sch_print',
-             'out_hpgl',
-             'out_dxf',
-             'out_pdf_pcb_print',
-             'out_pdf',
-             'out_svg',
-             'out_pcbdraw',
-             'pre_ignore_unconnected',
-             'pre_check_zone_fills',
-             'out_gerber',
+             'fil_base',
+             'fil_generic',
              'out_any_drill',
-             'out_step',
-             'out_ps',
-             'pre_drc',
-             'out_excellon',
-             'out_bom',
+             'out_any_layer',
              'out_base',
+             'out_bom',
+             'out_dxf',
+             'out_excellon',
+             'out_gerb_drill',
+             'out_gerber',
+             'out_hpgl',
              'out_ibom',
              'out_kibom',
+             'out_pcbdraw',
+             'out_pdf',
+             'out_pdf_pcb_print',
+             'out_pdf_sch_print',
              'out_position',
-             'layer',
-             ]
+             'out_ps',
+             'out_sch_variant',
+             'out_step',
+             'out_svg',
+             'out_svg_sch_print',
+             'pre_check_zone_fills',
+             'pre_drc',
+             'pre_erc',
+             'pre_filters',
+             'pre_ignore_unconnected',
+             'pre_update_xml',
+             'var_base',
+             'var_kibom',
+            ]
 
 
 def list_pre_and_outs(logger, outputs):
@@ -251,6 +258,14 @@ def main():
     GS.debug_enabled = logger.getEffectiveLevel() <= DEBUG
     GS.debug_level = args.verbose
 
+    # Parse global overwrite options
+    for redef in args.global_redef:
+        if '=' not in redef:
+            logger.error('Malformed global-redef option, must be VARIABLE=VALUE ({})'.format(redef))
+            sys.exit(EXIT_BAD_ARGS)
+        var = redef.split('=')[0]
+        GS.global_from_cli[var] = redef[len(var)+1:]
+
     clean_cache()
 
     # Output dir: relative to CWD (absolute path overrides)
@@ -300,8 +315,10 @@ def main():
     GS.set_sch(solve_schematic(args.schematic, args.board_file))
     # Determine the PCB file
     GS.set_pcb(solve_board_file(GS.sch_file, args.board_file))
-
+    # Do all the job (pre-flight + outputs)
     generate_outputs(outputs, args.target, args.invert_sel, args.skip_pre)
+    # Print total warnings
+    logger.log_totals()
 
 
 if __name__ == "__main__":
