@@ -57,6 +57,7 @@ def _import(name, path):
 def _load_actions(path):
     logger.debug("Importing from "+path)
     lst = glob(os.path.join(path, 'out_*.py')) + glob(os.path.join(path, 'pre_*.py'))
+    lst += glob(os.path.join(path, 'var_*.py')) + glob(os.path.join(path, 'fil_*.py'))
     for p in lst:
         name = os.path.splitext(os.path.basename(p))[0]
         logger.debug("- Importing "+name)
@@ -147,6 +148,8 @@ def load_board(pcb_file=None):
 
 
 def load_sch():
+    if GS.sch:  # Already loaded
+        return
     GS.kicad_version = pcbnew.GetBuildVersion()
     logger.debug('KiCad: '+GS.kicad_version)
     GS.check_sch()
@@ -168,6 +171,29 @@ def load_sch():
         exit(EXIT_BAD_CONFIG)
 
 
+def get_board_comps_data(comps):
+    if GS.board_comps_joined or not GS.pcb_file:
+        return
+    if not GS.board:
+        load_board()
+    comps_hash = {c.ref: c for c in comps}
+    for m in GS.board.GetModules():
+        ref = m.GetReference()
+        if ref not in comps_hash:
+            logger.warning('`{}` component in board, but not in schematic'.format(ref))
+            continue
+        c = comps_hash[ref]
+        attrs = m.GetAttributes()
+        # KiCad 5 (6 will change it)
+        if attrs == 1:
+            c.smd = True
+        elif attrs == 2:
+            c.virtual = True
+        else:
+            c.tht = True
+    GS.board_comps_joined = True
+
+
 def preflight_checks(skip_pre):
     logger.debug("Preflight checks")
 
