@@ -5,14 +5,16 @@
 # Project: KiBot (formerly KiPlot)
 import os
 import re
-from shutil import which
 from tempfile import (NamedTemporaryFile)
-from subprocess import (check_output, STDOUT, CalledProcessError)
+# Here we import the whole module to make monkeypatch work
+import subprocess
+import shutil
 from .misc import PCBDRAW, PCBDRAW_ERR, URL_PCBDRAW
 from .kiplot import check_script
 from .error import KiPlotConfigurationError
 from .gs import (GS)
-from .optionable import (BaseOptions, Optionable)
+from .optionable import Optionable
+from .out_base import VariantOptions
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
@@ -89,8 +91,8 @@ def _get_tmp_name(ext):
 def _run_command(cmd, tmp_remap=False, tmp_style=False):
     logger.debug('Executing: '+str(cmd))
     try:
-        cmd_output = check_output(cmd, stderr=STDOUT)
-    except CalledProcessError as e:
+        cmd_output = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
+    except subprocess.CalledProcessError as e:
         logger.error('Failed to run %s, error %d', cmd[0], e.returncode)
         if e.output:
             logger.debug('Output from command: '+e.output.decode())
@@ -103,7 +105,7 @@ def _run_command(cmd, tmp_remap=False, tmp_style=False):
     logger.debug('Output from command:\n'+cmd_output.decode())
 
 
-class PcbDrawOptions(BaseOptions):
+class PcbDrawOptions(VariantOptions):
     def __init__(self):
         with document:
             self.style = PcbDrawStyle
@@ -123,7 +125,7 @@ class PcbDrawOptions(BaseOptions):
             self.highlight = Optionable
             """ [list(string)=[]] list of components to highlight """
             self.show_components = Optionable
-            """ [string|list(string)=none] [none,all] list of components to draw, can be also a string for none or all.
+            """ [list(string)|string=none] [none,all] list of components to draw, can be also a string for none or all.
                 The default is none """
             self.vcuts = False
             """ render V-CUTS on the Cmts.User layer """
@@ -153,6 +155,8 @@ class PcbDrawOptions(BaseOptions):
         if isinstance(self.show_components, type):
             self.show_components = ''
         elif isinstance(self.show_components, str):
+            if self.variant or self.dnf_filter:
+                logger.warning('Ambiguous list of components to show `{}` vs variant/filter'.format(self.show_components))
             if self.show_components == 'none':
                 self.show_components = ''
             else:
@@ -203,7 +207,27 @@ class PcbDrawOptions(BaseOptions):
             f.close()
             return f.name
 
+    def _append_output(self, cmd, output):
+        svg = None
+        if self.format == 'svg':
+            cmd.append(output)
+        else:
+            # PNG and JPG outputs are unreliable
+            if shutil.which(SVG2PNG) is None:
+                logger.warning('`{}` not installed, using unreliable PNG/JPG conversion'.format(SVG2PNG))
+                logger.warning('If you experiment problems install `librsvg2-bin` or equivalent')
+                cmd.append(output)
+            elif shutil.which(CONVERT) is None:
+                logger.warning('`{}` not installed, using unreliable PNG/JPG conversion'.format(CONVERT))
+                logger.warning('If you experiment problems install `imagemagick` or equivalent')
+                cmd.append(output)
+            else:
+                svg = _get_tmp_name('.svg')
+                cmd.append(svg)
+        return svg
+
     def run(self, output_dir, board):
+        super().run(output_dir, board)
         check_script(PCBDRAW, URL_PCBDRAW, '0.6.0')
         # Output file name
         output = self.expand_filename(output_dir, self.output, 'bottom' if self.bottom else 'top', self.format)
@@ -230,6 +254,10 @@ class PcbDrawOptions(BaseOptions):
         if self.highlight:
             cmd.extend(['-a', self.highlight])
         if self.show_components is not None:
+            to_add = ','.join(self.get_fitted_refs())
+            if self.show_components and to_add:
+                self.show_components += ','
+            self.show_components += to_add
             cmd.extend(['-f', self.show_components])
         if self.vcuts:
             cmd.append('-v')
@@ -246,22 +274,7 @@ class PcbDrawOptions(BaseOptions):
             tmp_remap = None
         # The board & output
         cmd.append(GS.pcb_file)
-        svg = None
-        if self.format == 'svg':
-            cmd.append(output)
-        else:
-            # PNG and JPG outputs are unreliable
-            if which(SVG2PNG) is None:
-                logger.warning('`{}` not installed, using unreliable PNG/JPG conversion'.format(SVG2PNG))
-                logger.warning('If you experiment problems install `librsvg2-bin` or equivalent')
-                cmd.append(output)
-            elif which(CONVERT) is None:
-                logger.warning('`{}` not installed, using unreliable PNG/JPG conversion'.format(CONVERT))
-                logger.warning('If you experiment problems install `imagemagick` or equivalent')
-                cmd.append(output)
-            else:
-                svg = _get_tmp_name('.svg')
-                cmd.append(svg)
+        svg = self._append_output(cmd, output)
         # Execute and inform is successful
         _run_command(cmd, tmp_remap, tmp_style)
         if svg is not None:
