@@ -8,12 +8,16 @@
 import operator
 from datetime import datetime
 from pcbnew import (IU_PER_MM, IU_PER_MILS)
-from .optionable import BaseOptions
 from .gs import GS
+from .misc import UI_SMD, UI_VIRTUAL
+from .out_base import VariantOptions
 from .macros import macros, document, output_class  # noqa: F401
+from . import log
+
+logger = log.get_logger(__name__)
 
 
-class PositionOptions(BaseOptions):
+class PositionOptions(VariantOptions):
     def __init__(self):
         with document:
             self.format = 'ASCII'
@@ -120,6 +124,7 @@ class PositionOptions(BaseOptions):
             bothf.close()
 
     def run(self, output_dir, board):
+        super().run(output_dir, board)
         columns = ["Ref", "Val", "Package", "PosX", "PosY", "Rot", "Side"]
         colcount = len(columns)
         # Note: the parser already checked the units are milimeters or inches
@@ -129,14 +134,23 @@ class PositionOptions(BaseOptions):
         else:  # self.units == 'inches':
             conv = 0.001 / IU_PER_MILS
         # Format all strings
+        comps_hash = self.get_refs_hash()
         modules = []
         for m in sorted(board.GetModules(), key=operator.methodcaller('GetReference')):
-            if (self.only_smd and m.GetAttributes() == 1) or not self.only_smd:
+            ref = m.GetReference()
+            # Apply any filter or variant data
+            if comps_hash:
+                c = comps_hash.get(ref, None)
+                if c and (not c.fitted or not c.included):
+                    continue
+            # If passed check the position options
+            if (self.only_smd and m.GetAttributes() == UI_SMD) or \
+               (not self.only_smd and m.GetAttributes() != UI_VIRTUAL):
                 center = m.GetCenter()
                 # See PLACE_FILE_EXPORTER::GenPositionData() in
                 # export_footprints_placefile.cpp for C++ version of this.
                 modules.append([
-                    "{}".format(m.GetReference()),
+                    "{}".format(ref),
                     "{}".format(m.GetValue()),
                     "{}".format(m.GetFPID().GetLibItemName()),
                     "{:.4f}".format(center.x * conv),
