@@ -4,18 +4,20 @@
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 import re
+import os
 from subprocess import (check_output, STDOUT, CalledProcessError)
+from tempfile import NamedTemporaryFile
 from .error import KiPlotConfigurationError
 from .misc import (KICAD2STEP, KICAD2STEP_ERR)
 from .gs import (GS)
-from .optionable import BaseOptions
+from .out_base import VariantOptions
 from .macros import macros, document, output_class  # noqa: F401
 from . import log
 
 logger = log.get_logger(__name__)
 
 
-class STEPOptions(BaseOptions):
+class STEPOptions(VariantOptions):
     def __init__(self):
         with document:
             self.metric_units = True
@@ -42,7 +44,39 @@ class STEPOptions(BaseOptions):
             raise KiPlotConfigurationError('Origin must be `grid` or `drill` or `X,Y`')
         self._origin = val
 
+    def filter_components(self, dir):
+        if not self._comps:
+            return GS.pcb_file
+        comps_hash = self.get_refs_hash()
+        # Remove the 3D models for not fitted components
+        rem_models = []
+        for m in GS.board.GetModules():
+            ref = m.GetReference()
+            c = comps_hash.get(ref, None)
+            if c and c.included and not c.fitted:
+                models = m.Models()
+                rem_m_models = []
+                while not models.empty():
+                    rem_m_models.append(models.pop())
+                rem_models.append(rem_m_models)
+        # Save the PCB to a temporal file
+        with NamedTemporaryFile(mode='w', suffix='.kicad_pcb', delete=False, dir=dir) as f:
+            fname = f.name
+        logger.debug('Storing filtered PCB to `{}`'.format(fname))
+        GS.board.Save(fname)
+        # Undo the removing
+        for m in GS.board.GetModules():
+            ref = m.GetReference()
+            c = comps_hash.get(ref, None)
+            if c and c.included and not c.fitted:
+                models = m.Models()
+                restore = rem_models.pop(0)
+                for model in restore:
+                    models.push_front(model)
+        return fname
+
     def run(self, output_dir, board):
+        super().run(output_dir, board)
         # Output file name
         output = self.expand_filename(output_dir, self.output, '3D', 'step')
         # Make units explicit
@@ -64,7 +98,8 @@ class STEPOptions(BaseOptions):
         else:
             cmd.extend(['--user-origin', "{}{}".format(self.origin.replace(',', 'x'), units)])
         # The board
-        cmd.append(GS.pcb_file)
+        board_name = self.filter_components(GS.pcb_dir)
+        cmd.append(board_name)
         # Execute and inform is successful
         logger.debug('Executing: '+str(cmd))
         try:
@@ -76,6 +111,10 @@ class STEPOptions(BaseOptions):
             if e.output:
                 logger.debug('Output from command: '+e.output.decode())
             exit(KICAD2STEP_ERR)
+        finally:
+            # Remove the temporal PCB
+            if board_name != GS.pcb_file:
+                os.remove(board_name)
         logger.debug('Output from command:\n'+cmd_output.decode())
 
 
